#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>

#define EEPROM_VALUE    0

/**
 * PWM dimmer for LED lights - you're welcome to use this code, but please drop me a note
 * at captainunlikely.com to let me know if you do
 * 
 * PB0      - led
 * PB1      - main pwm
 * PB2/INT0 - button
 * PB3/ADC3 - rota
 * PB4/ADC2 - rotb
 * PB5/ADC0 - unconnected (reset pin, pulled high)
 */

const int8_t ROTARYSTATE[16] PROGMEM = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0};
volatile int16_t rotary;
volatile uint8_t buttonbounce;
volatile uint16_t rotarybounce, buttondownbounce;

void resetwatchdog(void) __attribute__((naked)) __attribute__((section(".init3")));
void resetwatchdog(void) {
    MCUSR = 0;
    wdt_disable();
}

void set(uint8_t pwm) {
    if (pwm == 0) {
        TCCR0A &= ~(1<<COM0B1);
        PORTB &= ~(1<<PB1);
    } else if (pwm == 255) {
        TCCR0A &= ~(1<<COM0B1);
        PORTB |= (1<<PB1);
    } else {
        TCCR0A |= (1<<COM0B1);
        OCR0B = pwm;
    }
}

void intled(uint8_t pwm) {
    if (pwm == 0) {
        TCCR0A &= ~(1<<COM0A1);
        PORTB &= ~(1<<PB0);
    } else if (pwm == 255) {
        TCCR0A &= ~(1<<COM0A1);
        PORTB |= (1<<PB0);
    } else {
        TCCR0A |= (1<<COM0A1);
        OCR0A = pwm;
    }
}

ISR(INT0_vect) {
    buttonbounce = 255;
}

ISR(PCINT0_vect) {
    rotarybounce = 1;
    static uint8_t v = 3;
    static int8_t rotval = 0;
    v <<=2;
    if (PINB & (1<<PB4)) {
        v |= 1;
    }
    if (PINB & (1<<PB3)) {
        v |= 2;
    }
    rotval += pgm_read_byte(&(ROTARYSTATE[v & 0x0f]));
    if (rotval > 3) {
        rotary++;
        rotval = 0;
    } else if (rotval < -3) {
        rotary--;
        rotval = 0;
    }
}


const uint8_t steps[] = { 1, 4, 8, 12, 16, 24, 32, 30, 48, 56, 64, 80, 96, 112, 128, 140, 160, 192, 208, 255 };

int main(void) {

    set(0);
    intled(255);
    _delay_ms(1000);
    intled(0);
    DDRB |= 1<<DDB0;                                    // LED
    DDRB |= 1<<DDB1;                                    // PWM
    PORTB |= (1<<PB2) | (1<<PB3) | (1<<PB4);            // Button Pullup
//    TCCR0B |= (1<<CS00);                              // CLK/1/256 = 3906Hz @ 1Mhz - audible hum, bad!
    TCCR0B |= (1<<CS01);                                // CLK/8/256 = 488Hz @ 1Mhz
    TCCR0A |= (1<<WGM01) | (1<<WGM00);
    GIMSK |= (1<<INT0) | (1<<PCIE);
    PCMSK |= (1<<PCINT3) | (1<<PCINT4);

    uint8_t on = 0;
    uint8_t value = eeprom_read_byte(EEPROM_VALUE);
    uint8_t intvalue = 0;
    if (value >= sizeof(steps)) {
        value = sizeof(steps) / 2;
    }

    sei();

    while(1) {
        uint8_t buttonup = PINB & (1<<PB2);
        if (buttonbounce && !--buttonbounce) {
            if (buttonup) {
                buttondownbounce = 0;
                on = !on;
                if (on) {
                    intled(intvalue = 0);
                }
                if (!on && rotarybounce) {
                    if (eeprom_read_byte(EEPROM_VALUE) != value) {
                        eeprom_write_byte(EEPROM_VALUE, value);
                    }
                    rotarybounce = 0;
                }
                if (value >= sizeof(steps)) {
                    value = sizeof(steps) - 1;
                }
                set(on ? steps[value]: 0);
            } else {
                intled(255);
            }
        } else {
            cli();
            int16_t v = rotary;
            rotary = 0;
            sei();
            if (v) {
                if (on) {
                    if (v > 0 && value < sizeof(steps) - 1) {
                        value++;
                    } else if (v < 0 && value > 0) {
                        value--;
                    }
                    if (value >= sizeof(steps)) {
                        value = sizeof(steps) - 1;
                    }
                    set(steps[value]);
                } else {
                    if (v > 0 && intvalue < sizeof(steps) - 1) {
                        intvalue++;
                    } else if (v < 0 && intvalue > 0) {
                        intvalue --;
                    }
                    if (intvalue >= sizeof(steps)) {
                        intvalue = sizeof(steps) - 1;
                    }
                    intled(intvalue);
                }
            }
            if (rotarybounce && !++rotarybounce) {
                if (eeprom_read_byte(EEPROM_VALUE) != value) {
                    eeprom_write_byte(EEPROM_VALUE, value);
                }
            }
        }
        if (!rotarybounce && !buttonbounce) {
            set_sleep_mode(on || intvalue ? SLEEP_MODE_IDLE : SLEEP_MODE_PWR_DOWN);
            sleep_mode();
            sleep_cpu();
            sleep_disable();
        }
    }
}
